

%-------------------------------------------------
% Calcolo del Call/Put secondo il modello di CRR
%-------------------------------------------------

%-------------------------------------------------
% Input
%-------------------------------------------------
% Delta_t: intervallo di tempo che intercorre tra
%          due osservazioni consecutive.
% N: numero di periodi 
% sigma: volatilit
% K: Strike
% r: tasso risk-free
% S0: prezzo spot al tempo t
% c: variabile di controllo
%    c=1 viene costruito il grafico dell'albero
%        binomiale per il sottostante
%
%-------------------------------------------------

%-------------------------------------------------
% output
%-------------------------------------------------
% yc: prezzo dell'opzione call europea
% yp: prezzo dell'opzione put europea
%-------------------------------------------------


function [yc,yp]=Tree(Delta_t,N,sigma,K,r,S0,c)

% Modello CRR


u=exp(sigma.*sqrt(Delta_t));

d=1./u;

p=(exp(r.*Delta_t)-d)./(u-d);


% Definizione dei possibili valori finali del sottostante

% i: indice di tempo, 1<=i<=N
% j: indice dei possibili valori del prezzo del sottostante al tempo i, 1<=j<=(i+1)


for i=1:1:N
    
        for j=1:1:(i+1)
            
            S(j,i)=S0.*(u.^(j-1)).*(d.^(i-j+1));

        end
        
end        


% calcolo del payoff

for j=1:1:(N+1)
    
    
    fp(j,N)=max(K-S(j,N),0);
    
    fc(j,N)=max(S(j,N)-K,0);
    
end


if (N==1)
    
    
    yp=p.*fp(2,N)+(1-p).*fp(1,N);
    
    yc=p.*fc(2,N)+(1-p).*fc(1,N);
    
    
else

for i=(N-1):-1:1
    
        for j=(i+1):-1:1


         fp(j,i)=(p.*fp(j+1,i+1)+(1-p).*fp(j,i+1)).*exp(-r.*Delta_t);   
            
         fc(j,i)=(p.*fc(j+1,i+1)+(1-p).*fc(j,i+1)).*exp(-r.*Delta_t);   
            
        end
        
end        


    yp=p.*fp(2,1)+(1-p).*fp(1,1);
    
    yc=p.*fc(2,1)+(1-p).*fc(1,1);


end    

if (c==1)

        

figure;

hold on

   line([0,Delta_t], [S0,S0.*u]);

   line([0,Delta_t], [S0,S0.*d]);
 
   

  for i=1:1:(N-1)
    
        for j=1:1:(i+1)


         line([i.*Delta_t,(i+1).*Delta_t], [S(j,i),S(j,i).*u]);

         line([i.*Delta_t,(i+1).*Delta_t], [S(j,i),S(j,i).*d]);

        end
        
  end

end





        
            
            
            
            
            